#!/usr/bin/env python
# 
# Indentation finder, by Philippe Fremy <phil at freehackers dot org>
# Copyright 2002,2005 Philippe Fremy
#
# This program is distributed under the BSD license. You should have received
# a copy of the file LICENSE.txt along with this software.
#

from indent_checker import *
from indent_finder import *

import os
from unittest import *

fa = [ 'file1.a', 'file2.a' ]
fb = [ 'file1.b', 'file2.b' ]
tfa = [ os.path.join( 'tests', f ) for f in fa ] 
tfb = [ os.path.join( 'tests', f ) for f in fb ] 
file_list = fa + fb + tfa + tfb 

class Test_Indent_Checker( TestCase ):

	def setUp( self ):
		for f in file_list:
			open(f,'w').write('some content\n')

		for f in file_list:
			self.assertEquals( os.path.exists( f ), True )

	def tearDown( self ):
		for f in file_list:
			if os.path.exists( f ): os.unlink( f )

		for f in file_list:
			self.assertEquals( os.path.exists( f ), False )

	def test_build_file_list( self ):
		file_list = build_file_list( [ 'tests' ], ['*.a'], True )
		self.assertEquals( file_list, tfa )

		file_list = build_file_list( [ 'tests' ], ['*.b'], True )
		self.assertEquals( file_list, tfb )

		file_list = build_file_list( [ 'tests' ], ['*.a', '*.b'], True )
		self.assertEquals( file_list, tfa + tfb )

		file_list = build_file_list( [ '.' ], ['*.a'], True )
		self.assertEquals( file_list, fa + tfa )

		file_list = build_file_list( [ '.' ], ['*.b'], True )
		self.assertEquals( file_list, fb + tfb )

		file_list = build_file_list( [ '.' ], ['*.a', '*.b'], True )
		self.assertEquals( file_list, file_list )

		file_list = build_file_list( [ '.' ], ['*.a', '*.b'], False )
		self.assertEquals( file_list, fa + fb )

	def test_find_bad_indentation( self ):
		tuple_list = [ 
			( 'IOtest.java', 	'space 4' ),
			( 'cml.py', 		'space 4' ),
			( 'pretty-make.py', 'tab 8' ),
			( 'DebugClient.py',	'space 4' ),
			( 'TestRunner.cpp', 'space 2' ),
			( 'diffmodel.cpp', 	'tab 8' ),
		]
		tuple_list = [ (os.path.join( 'tests', f[0] ),f[1]) for f in tuple_list ]
		file_list = [ f[0] for f in tuple_list ]
		print file_list

		expected = [ f for f in tuple_list if f[1] != 'space 4' ]
		expected.sort()
		result = find_bad_indentation( file_list, INDENT_SPACE_4 )
		result.sort()
		self.assertEquals( result, expected )

		expected = [ f for f in tuple_list if f[1] != 'space 2' ]
		expected.sort()
		result = find_bad_indentation( file_list, INDENT_SPACE_2 )
		result.sort()
		self.assertEquals( result, expected )

		expected = [ f for f in tuple_list if f[1] != 'tab 8' ]
		expected.sort()
		result = find_bad_indentation( file_list, INDENT_TAB )
		result.sort()
		self.assertEquals( result, expected )


if __name__ == "__main__":
	main( testRunner = TextTestRunner( verbosity = 2 ) )
