#!/usr/bin/env python
# 
# Indentation finder, by Philippe Fremy <phil at freehackers dot org>
# Copyright 2002,2005 Philippe Fremy
#
# This program is distributed under the BSD license. You should have received
# a copy of the file LICENSE.txt along with this software.
#


from indent_finder import *

import os
from unittest import *

class Test_find_indent( TestCase ):

	def test_indent_re( self ):
		indentFinder = IndentFinder()
		self.assertEquals( indentFinder.nb_skipped_lines, 0 )

		indentFinder.clear()
		indentFinder.analyse_line( "\t\t  hop" ) # skip mixed indentation
		self.assertEquals( indentFinder.nb_skipped_lines, 1 )
		self.assertEquals( indentFinder.nb_lines, 1 )

		indentFinder.clear()
		indentFinder.analyse_line( "\t\t  " ) # skip blank line
		self.assertEquals( indentFinder.nb_skipped_lines, 1 )
		self.assertEquals( indentFinder.nb_lines, 1 )

		indentFinder.clear()
		indentFinder.analyse_line( "\t\thop\\\n" )
		indentFinder.analyse_line( "\t\tbof" ) # only one line when last char is \
		self.assertEquals( indentFinder.nb_skipped_lines, 1 )
		self.assertEquals( indentFinder.nb_lines, 2 )

		indentFinder.clear()
		indentFinder.analyse_line( "\t\thop" ) # take tab
		self.assertEquals( indentFinder.nb_skipped_lines, 0 )
		self.assertEquals( indentFinder.nb_lines, 1 )

		indentFinder.clear()
		indentFinder.analyse_line( "    hop" ) # take spaces
		self.assertEquals( indentFinder.nb_skipped_lines, 0 )
		self.assertEquals( indentFinder.nb_lines, 1 )

		indentFinder.clear()
		indentFinder.analyse_line( "    hop" ) # 4 spaces
		self.assertEquals( indentFinder.nb_skipped_lines, 0 )
		self.assertEquals( indentFinder.nb_lines, 1 )
		self.assertEquals( indentFinder.spaces[2-2], 1 )
		self.assertEquals( indentFinder.spaces[4-2], 1 )
		indentFinder.analyse_line( "        hop" ) # 8 spaces
		self.assertEquals( indentFinder.nb_skipped_lines, 0 )
		self.assertEquals( indentFinder.nb_lines, 2 )
		self.assertEquals( indentFinder.spaces[2-2], 2 )
		self.assertEquals( indentFinder.spaces[4-2], 2+10 ) # bonus
		indentFinder.analyse_line( "        hop" ) # 8 spaces again
		self.assertEquals( indentFinder.nb_skipped_lines, 0 )
		self.assertEquals( indentFinder.nb_lines, 3 )
		self.assertEquals( indentFinder.spaces[2-2], 3 )
		self.assertEquals( indentFinder.spaces[4-2], 3+10 ) # no bonus

	def check_file( self, fname, result ):
		indentFinder = IndentFinder()
		indentFinder.parse_file( os.path.join( "tests", fname  ) )
		res = str(indentFinder)
		self.assertEquals( res, result )

	def test_debug_client_py( self ):
		self.check_file( "DebugClient.py", INDENT_SPACE_4 )

	def test_diff_model_cpp( self ):
		self.check_file( "diffmodel.cpp", INDENT_TAB )

	def test_iotest_java( self ):
		self.check_file( "IOtest.java", INDENT_SPACE_4 )

	def test_pretty_make_py( self ):
		self.check_file( "pretty-make.py", INDENT_TAB )

	def test_test_runner_py( self ):
		self.check_file( "TestRunner.cpp", INDENT_SPACE_2 )

	def test_cml_py( self ):
		self.check_file( "cml.py", INDENT_SPACE_4 )

if __name__ == "__main__":
	main( testRunner = TextTestRunner( verbosity = 2 ) )
